/**
 *
 * Program name: GetMatchNone
 *
 * Description: Simple C program that issues an MQGET for a
 *              non-existent message.  Queues with large message
 *              count may cause the queue manager to output
 *              error message 'AMQ7234'. See
 *  https://www.ibm.com/support/pages/amq7234-issued-periodically-wmq
 *              This program will keep the queue manager from
 *              unloading the messages.
 *
 * Parameters:  QueueName and QMgrName
 *
 * Sample Service Definition:
 *
     DEFINE SERVICE ('GetMatchNone') +
            DESCR('Keep QMgr from unloading the messages of this queue.') +
            STARTCMD('C:\Capitalware\Utils\GetMatchNone.exe') +
            STARTARG('{QueueName} +QMNAME+') +
            STOPCMD(' ') +
            STOPARG(' ') +
            STDOUT('C:\Capitalware\Utils\stdout.log') +
            STDERR('C:\Capitalware\Utils\stderr.log') +
            CONTROL(STARTONLY) +
            SERVTYPE(SERVER) +
            REPLACE
 *
 */

#if defined(WIN32)
#pragma warning( disable : 4996 )
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

/* includes for MQI  */
#include <cmqc.h>

#define LOG_RECORD_FORMAT_STRING "%d/%02d/%02d %02d:%02d:%02d %s Line#%0.5d: %s\n"

#define CRAZY_CORREL_ID "Non-Existent crazy Correlation Id"

/*
 * Logging message labels
 */
static char * LOG_TYPE_INFO   = "INFO   "; /* General informational log msg */
static char * LOG_TYPE_WARNING= "WARNING"; /* Warning messages */
static char * LOG_TYPE_ERROR  = "ERROR  "; /* Error messages - show stoppers */

/* prototypes  */
void LogIt( char *, int, char *);

int main(int argc, char **argv)
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   MQOD     od  = {MQOD_DEFAULT};   /* Object Descriptor             */
   MQMD     md  = {MQMD_DEFAULT};   /* Message Descriptor            */
   MQGMO    gmo = {MQGMO_DEFAULT};  /* get message options           */

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle                 */
   MQLONG   O_options = MQOO_BROWSE | MQOO_FAIL_IF_QUIESCING; /* MQOPEN options */
   MQLONG   C_options = MQCO_NONE;  /* close options - none          */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   OpenCode;               /* MQOPEN completion code        */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason code for MQCONN        */
   char     Buffer[2] = " ";        /* point to buffer               */
   MQLONG   buflen = 1;
   MQLONG   messlen;                /* message length received       */
   char     QMName[50];             /* queue manager name            */
   char     TempString[1024];

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   LogIt(LOG_TYPE_INFO,  __LINE__, "GetMatchNone starting");
   if (argc < 3)
   {
      LogIt(LOG_TYPE_ERROR,  __LINE__, "Required parameter missing - QName QMgrName\n");
      exit(99);
   }

   /* Create object descriptor for subject queue & QMgr */
   strcpy(od.ObjectName, argv[1]);
   strcpy(QMName, argv[2]);

   /* Connect to queue manager */
   MQCONN(QMName,                  /* queue manager                  */
          &Hcon,                   /* connection handle              */
          &CompCode,               /* completion code                */
          &CReason);               /* reason code                    */

   sprintf(TempString,
           "MQCONN  CC=%d RC=%d for %s",
           CompCode,
           CReason,
           argv[2]);

   /* report reason and stop if it failed. */
   if (CompCode == MQCC_FAILED)
   {
      LogIt(LOG_TYPE_ERROR, __LINE__, TempString);
      exit( (int)CReason );
   }
   else
      LogIt(LOG_TYPE_INFO, __LINE__, TempString);

   MQOPEN(Hcon,                      /* connection handle            */
          &od,                       /* object descriptor for queue  */
          O_options,                 /* open options                 */
          &Hobj,                     /* object handle                */
          &OpenCode,                 /* completion code              */
          &Reason);                  /* reason code                  */

   sprintf(TempString,
           "MQOPEN  CC=%d RC=%d for %s",
           OpenCode,
           Reason,
           argv[1]);

   /* report reason, if any; stop if failed      */
   if (Reason != MQRC_NONE)
      LogIt(LOG_TYPE_ERROR, __LINE__, TempString);
   else
      LogIt(LOG_TYPE_INFO, __LINE__, TempString);

   if (OpenCode != MQCC_FAILED)
   {
      gmo.Options = MQGMO_FAIL_IF_QUIESCING | MQGMO_WAIT | MQGMO_BROWSE_NEXT;
      gmo.WaitInterval = MQWI_UNLIMITED;
      gmo.MatchOptions = MQMO_MATCH_CORREL_ID;

      memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
      memcpy(md.CorrelId, CRAZY_CORREL_ID, sizeof(md.CorrelId));

      MQGET(Hcon,                /* connection handle                 */
            Hobj,                /* object handle                     */
            &md,                 /* message descriptor                */
            &gmo,                /* get message options               */
            buflen,              /* Buffer length                     */
            Buffer,              /* pointer to message buffer         */
            &messlen,            /* message length                    */
            &CompCode,           /* completion code                   */
            &Reason);            /* reason code                       */

      sprintf(TempString,
              "MQGET   CC=%d RC=%d for %s",
              CompCode,
              Reason,
              argv[1]);

      if (Reason != MQRC_NONE)
         LogIt(LOG_TYPE_ERROR, __LINE__, TempString);
      else
         LogIt(LOG_TYPE_INFO, __LINE__, TempString);

      /* Close the queue */
      MQCLOSE(Hcon,                   /* connection handle           */
              &Hobj,                  /* object handle               */
              C_options,
              &CompCode,              /* completion code             */
              &Reason);               /* reason code                 */

      sprintf(TempString,
              "MQCLOSE CC=%d RC=%d for %s",
              CompCode,
              Reason,
              argv[1]);

      if (Reason != MQRC_NONE)
         LogIt(LOG_TYPE_ERROR, __LINE__, TempString);
      else
         LogIt(LOG_TYPE_INFO, __LINE__, TempString);
   }

   /* Disconnect from MQM if not already connected */
   if (CReason != MQRC_ALREADY_CONNECTED )
   {
      MQDISC(&Hcon,                     /* connection handle          */
             &CompCode,                 /* completion code            */
             &Reason);                  /* reason code                */

      sprintf(TempString,
              "MQDISC  CC=%d RC=%d for %s",
              CompCode,
              Reason,
              argv[2]);

      if (Reason != MQRC_NONE)
         LogIt(LOG_TYPE_ERROR, __LINE__, TempString);
      else
         LogIt(LOG_TYPE_INFO, __LINE__, TempString);
   }

   LogIt(LOG_TYPE_INFO,  __LINE__, "GetMatchNone ending\n");
   return(0);
}


void LogIt( char *severity, int line, char *msg )
{
   /* --------------------------------------------
    * Variable declarations.
    * --------------------------------------------
    */
   struct tm*      CurrentTime;
   time_t          LongTime;

   /* --------------------------------------------
    * Code section
    * --------------------------------------------
    */
   /* Get the current (local) time. */
   LongTime = time(NULL);
   CurrentTime = localtime(&LongTime);

   printf(LOG_RECORD_FORMAT_STRING,
          (CurrentTime->tm_year + 1900),
          (CurrentTime->tm_mon + 1),
          CurrentTime->tm_mday,
          CurrentTime->tm_hour,
          CurrentTime->tm_min,
          CurrentTime->tm_sec,
          severity,
          line,
          msg);

   return;
}
